#!/usr/bin/perl

#
# $Id: bios_m8like2compendiumreport.pl 298 2010-04-19 09:49:20Z carrere $
#

# Copyright INRA

# http://cati-bipas.toulouse.inra.fr/

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

bios_m8like2compendiumreport.pl

=head1 SYNOPSIS

=head1 DESCRIPTION

Converts m8 like reports (from SOAP or blast mapping programs) to Compendium compliant report file.

=cut

BEGIN
{
    use Cwd 'abs_path';
    use File::Basename;
    my ($dirprg) = dirname(abs_path($0));
    unshift(@INC, "$dirprg/../../playmoby/lib");
    unshift(@INC, "$dirprg/../../lib");
    $ENV{BIOS} = "$dirprg/../.." if (!defined($ENV{BIOS}));

    our $VERSION = do {my @r = (q$Rev: 298 $ =~ /\d+/g); $r[0]};
}

use strict;
use warnings;
use Carp;
use ParamParser;
use IO::File;
use DBI;
use Cwd 'abs_path';
use File::stat;
use Time::localtime;
use File::Basename;

require "BIOS/Utils.pl";

#Initialisation des valeurs pour le Profiler
my $CFG = $ENV{BIOS} . "/site/cfg/bios.cfg";

MAIN:
{

    my $o_param = New ParamParser('GETOPTLONG', ('m8_files=s', 'outfile=s', 'verbose'));

    $o_param = &BIOSLoadCfg($o_param);

    $o_param->SetUsage(my $usage = sub {&Usage($o_param);});
    $o_param->AssertDefined('m8_files');

    my $m8_files   = $o_param->Get('m8_files');
    my $ra_infiles = &GetInfiles($m8_files);
    my $rh_members = {};

    foreach my $infile (@{$ra_infiles})
    {
        print STDERR "Infile : $infile\n" if ($o_param->IsDefined('verbose'));
        if (my $fh_m8file = &IsM8($infile, $o_param))
        {
            print STDERR "M8 File: $infile\n" if ($o_param->IsDefined('verbose'));
            $rh_members = &ParseM8File($fh_m8file, $rh_members);
            $fh_m8file->close;
        }
    }
    &PrintReport($rh_members, $o_param);

}

sub GetInfiles
{
    my $m8_files_param = shift;
    my @a_infiles      = ();
    if ($m8_files_param =~ /([\s,])/)
    {
        @a_infiles = split(/$1/, $m8_files_param);
    }
    else
    {
        if (my $ra_infiles = &IsFof($m8_files_param))
        {
            @a_infiles = @{$ra_infiles};
        }
        else
        {
            @a_infiles = ($m8_files_param);
        }
    }
    return \@a_infiles;
}

sub IsFof
{
    my $infile    = shift;
    my $fh_infile = new IO::File($infile) or Carp::croak("ERROR: Can not open $infile");
    my @a_infiles = ();
    if (defined $fh_infile)
    {

        while (my $line = <$fh_infile>)
        {
            chomp $line;
            if (!-e $line)
            {
                $fh_infile->close;
                return 0;
            }
            else
            {
                push(@a_infiles, $line);
            }

        }
        $fh_infile->close;
        return \@a_infiles;
    }
    else
    {
        return 0;
    }
    return 0;
}

sub IsM8
{
    my $infile    = shift;
    my $o_param   = shift;
    my $fh_infile = new IO::File($infile) or Carp::croak("ERROR: Can not open $infile");
    if (defined $fh_infile)
    {

        #Rosa-DBO#17358675#1#KN-1367_2_5_25_1612_1092	RC000004	100	32	2	0	1	32	44	13	1e-0	30	-
        #Rosa-DBO#17359104#1#KN-1367_2_5_17_897_1891	RC000009	100	32	0	0	1	32	550	519	1e-0	32	-
        #Rosa-DBO#17359115#1#KN-1367_2_5_17_341_1454	RC000004	100	32	0	0	1	32	577	546	1e-0	32	-

        my $first_line = <$fh_infile>;
        chomp $first_line;
        print STDERR "Testing line >$first_line<\n" if ($o_param->IsDefined('verbose'));
        return 0
#          if ($first_line !~ /^\S+\s+\S+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+\d+\s+[\de\-\.]+\s+\d+\s+[\+\-]$/);
	   if ($first_line !~ /^\S+\s+\S+\s+\-?\d+\s+\-?\d+\s+\-?\d+\s+\-?\d+\s+\-?\d+\s+\-?\d+\s+\-?\d+\s+\-?\d+\s+[\de\-\.]+\s+-?\d+\s+[\+\-]/);
        return $fh_infile;
    }
    return 0;

}

sub ParseM8File
{
    my $fh_m8file  = shift;
    my $rh_members = shift;
    while (my $line = <$fh_m8file>)
    {
        chomp $line;
        next if ($line eq '');

        my ($read, $cluster) = ($line =~ /^(\S+)\s+(\S+)\s+/);
        if (defined $cluster)
        {
            $rh_members->{$cluster} = [] if (!defined($rh_members->{$cluster}));
            push(@{$rh_members->{$cluster}}, $read);
        }
    }

    return $rh_members;

}

sub PrintReport
{
    my $rh_members = shift;
    my $o_param    = shift;

    my $fh_outfile = *STDOUT;
    if ($o_param->IsDefined('outfile'))
    {
        $fh_outfile = new IO::File('>' . $o_param->Get('outfile'))
          or Carp::croak("ERROR: Can not open " . $o_param->Get('outfile'));
    }

    foreach my $cluster (keys %{$rh_members})
    {
        print $fh_outfile ">$cluster\n" . join(' ', @{$rh_members->{$cluster}}) . "\n";
    }

    $fh_outfile->close;
    return;
}

sub Usage
{
    my $o_param = shift;

    print STDOUT <<END;
    $0
    
Mandatory

    --m8_files=<list of files, fof>

    --outfile=<compendium compliant file>

            
END

}

=pod

=head2 InterPro .desc format

    ID   Mext_0011
    AC   Mext_0011
    DE   AutoIPR: IPR010074:Phosphoribosylformylglycinamidine synthase II; IPR010077:Herpes virus tegument protein/v-FGAM-synthase;
    CC   AutoIPR_Raw: IPR000728:AIR synthase related protein; IPR010918:AIR synthase related protein, C-terminal;
    CC   AutoIPR_Family: IPR010074:Phosphoribosylformylglycinamidine synthase II; IPR010077:Herpes virus tegument protein/v-FGAM-synthase; 
    CC   AutoGO_Raw: IPR000728; Molecular Function: phosphoribosylformylglycinamidine synthase activity (GO:0004642), Cellular Component: cytoplasm (GO:0005737);
    CC   number of ipr DE/RAW: 2/4
    //
    ID   Mext_0012
    AC   Mext_0012
    DE   AutoIPR: IPR002634:BolA-like protein;
    CC   AutoIPR_Raw: IPR002634:BolA-like protein
    CC   AutoIPR_Family: IPR002634:BolA-like protein; 
    CC   number of ipr DE/RAW: 1/1
    //

=cut
