#!/usr/bin/perl

#
# $Id$
#                                                                       

# Copyright INRA

# http://cati-bipas.toulouse.inra.fr/
# Jerome.Gouzy@toulouse.inra.fr      

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL     
# "http://www.cecill.info".                                             

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

bios_mapreads.pl

=head1 SYNOPSIS

$BIOS/bin/int/bios_mapreads.pl --lib fastafile --reference fastafile --program soap|blastn --keep all|best|single [--cfg filename] [--program_param string] 

=head1 DESCRIPTION

--program soap 	for solexa reads
--keep all		report all hits
--keep single	report only hits mapped once
--keep best		report only best hit (ex: minimum number of MM for soap) but a hit is not reported when mapped several times with the same score

TODO

--program megablast for long reads

=cut


BEGIN
{
    use Cwd 'abs_path';
    use File::Basename;
    my ($dirprg) = dirname(abs_path($0));
    unshift(@INC, "$dirprg/../../lib");
    $ENV{BIOS} = "$dirprg/../.." if (!defined($ENV{BIOS}));
    our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

use Getopt::Long;

use warnings;
use strict;
use ParamParser;
use Carp;
use BerkeleyDB;

require "BIOS/Utils.pl";

my $CFG   = $ENV{BIOS} . "/site/cfg/bios.cfg";
my $DEBUG = 0;

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG', "lib=s", "reference=s", "program=s", "cfg=s", "program_param=s", "help","outfile=s","keep=s","overwrite","debug");

	$DEBUG = 1 if ($o_param->IsDefined('debug'));
	$o_param = &BIOSLoadCfg($o_param);

    $o_param->SetUsage(my $usage = sub {&Usage();});

    if ($o_param->IsDefined('cfg'))
    {
		$o_param->AssertFileExists('cfg');
        $o_param->Update($o_param->Get('cfg'), 'A');
    }

	$o_param->SetUnlessDefined('keep','best');
	
	$o_param->AssertDefined('outfile');
	$o_param->AssertAllowedValue('program','soap','blastn');
	if ( $o_param->Get('program') eq 'soap' )
	{
		$o_param->AssertFileExists('prg_soap','prg_soapindex');
		&MapWithSoap($o_param);
	}
	elsif ( $o_param->Get('program') eq 'blastn' )
	{
		$o_param->AssertFileExists('prg_blastn','prg_blastindex');
		&MapWithBlast($o_param);
	}
}


sub MapWithBlast
{
my($o_param) = @_;

	my $ref = $o_param->Get('reference');
	my @a_ls = `ls $ref.n?? 2> /dev/null`;
	if ( $#a_ls == -1 )
	{
		my $prg_index = $o_param->Get('prg_blastindex');
		my $cde = "$prg_index -in $ref -dbtype nucl -hash_index";
		print STDERR "$cde\n" if ( $DEBUG );
		`$cde`;
	}
	my $blastparam = ( $o_param->IsDefined('program_param') ) ?  $o_param->Get('program_param') : $o_param->Get('prg_blastparam');
	my $blastn 	  = $o_param->Get('prg_blastn');
	my $lib 	  = $o_param->Get('lib');
	my $out 	  = $o_param->Get('outfile');
	my $raw = "$out.$blastparam.raw";	
	$raw =~ s/\s//g;
	#my $cde = "$blastn -db $ref -query $lib $blastparam -out $raw -task megablast -outfmt 6 -perc_identity 95 -evalue 1e-6 -word_size 18 -dust no -best_hit_overhang 0.1 -parse_deflines";
	my $cde = "$blastn -db $ref -query $lib $blastparam -out $raw -task megablast -outfmt 6 -perc_identity 95 -evalue 1e-6 -word_size 18 -parse_deflines";
	if ( ! ( -e $raw && ! -z $raw ) || $o_param->Get('overwrite') )
	{
		print STDERR "$cde\n" if ( $DEBUG );
		`$cde`;
	}
	&FilterBlastN("$raw","$out",$o_param->Get('keep'));
}


sub FilterBlastN
{
my($raw,$result,$keep)=@_;

	tie my %h_hits, "BerkeleyDB::Hash";
	#my %h_hits=();
	my $f_raw = new IO::File($raw) or Carp::croak("ERROR: Can't read >$raw<\n");
	my $f_res='';
	if ( $DEBUG )
	{
		$f_res = new IO::File("$raw.$keep","w") or Carp::croak("ERROR: Can't create >$raw.$keep<\n");
	}
	else
	{
		$f_res = new IO::File("$result.$keep","w") or Carp::croak("ERROR: Can't create >$result.$keep<\n");
	}
	while(my $l=<$f_raw>)
	{
		chomp($l);
		my @a_ = split(' ',$l);
		&AnalyseHit('blastn',$l,\@a_,$f_res,\%h_hits,$keep);
	}
	$f_raw->close;
	if ( $keep ne 'all' )
	{
		&DumpHits($f_res,\%h_hits);
	}
	$f_res->close;
}


sub MapWithSoap
{
my($o_param) = @_;

	my $ref = $o_param->Get('reference');
	my $refindex = ( $ref !~ /\.index/ ) ? "$ref.index" : $ref;
	my @a_ls = `ls $refindex.*`;
	if ( $#a_ls == -1 )
	{
		my $soapindex = $o_param->Get('prg_soapindex');
		my $cde = "$soapindex $ref";
		print STDERR "$cde\n" if ( $DEBUG );
		`$cde`;
		$refindex = "$ref.index";
	}
	my $soapparam = ( $o_param->IsDefined('program_param') ) ?  $o_param->Get('program_param') : $o_param->Get('prg_soapparam');
	my $soap 	  = $o_param->Get('prg_soap');
	my $lib 	  = $o_param->Get('lib');
	my $out 	  = $o_param->Get('outfile');
	my $raw = "$out.$soapparam.raw";	
	$raw =~ s/\s//g;
	my $cde = "$soap -D $refindex -a $lib $soapparam -o $raw -u $raw.unmapped";
	if ( ! ( -e $raw && ! -z $raw ) || $o_param->Get('overwrite') )
	{
		print STDERR "$cde\n" if ( $DEBUG );
		`$cde`;
	}
	&FilterSoapAndConvertM8("$raw","$out",$o_param->Get('keep'));

}



sub FilterSoapAndConvertM8
{
my($raw,$result,$keep)=@_;

	tie my %h_hits, "BerkeleyDB::Hash";
	#my %h_hits=();
	my $f_raw = new IO::File($raw) or Carp::croak("ERROR: Can't read >$raw<\n");
	my $f_res='';
	if ( $DEBUG )
	{
		$f_res = new IO::File("$raw.$keep","w") or Carp::croak("ERROR: Can't create >$raw.$keep<\n");
	}
	else
	{
		$f_res = new IO::File("$result.$keep","w") or Carp::croak("ERROR: Can't create >$result.$keep<\n");
	}
	while(my $l=<$f_raw>)
	{
	#Rosa-BFL#17370433#1#KN-1367_5_6_72_684_447      GCCTGGGTGCTCGACAAGCTTAAGGCTGAGCG        hhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhh        1       a       32      -       RC000036        253     1       C->26T40        32M     26C5
		chomp($l);
		my @a_ = split(' ',$l);
        my $strand = $a_[6];
        my $aln = ($a_[5]-$a_[3]+1);
        my ($start,$end) = ($strand eq '+' ) ? ( $a_[8], $a_[8]+$aln-1 ) : ( $a_[8]+$aln-1, $a_[8] );
        my $pci = sprintf("%.0f",$aln*100/length($a_[1]));
        my $mismatch = length($a_[1]) - $aln + $a_[9];
		my $score    = length($a_[1]) - $mismatch;
		my $str = "$a_[0]\t$a_[7]\t$pci\t$aln\t$mismatch\t0\t$a_[3]\t$a_[5]\t$start\t$end\t1e-0\t$score\t$strand";
		&AnalyseHit('soap',$str,\@a_,$f_res,\%h_hits,$keep);
	}
	$f_raw->close;
	if ( $keep ne 'all' )
	{
		&DumpHits($f_res,\%h_hits);
	}
	$f_res->close;
}



sub DumpHits
{
my($f_res,$rh_hits)=@_;

	foreach my $read (sort keys(%$rh_hits))
	{
		next if ($$rh_hits{$read} =~ /MULTIPLEHITS/ );
		print $f_res "$$rh_hits{$read}\n";
	}
}


# j'externalise car l'analyse devrait etre commune aux differentes methodes de mapping
# pour soap, lorsque l'option -M vaut 4, tout les hits reportes ont le meme score donc apres notre filtre single == best, car il ne peut pas y avoir de meilleur

sub AnalyseHit
{
my($program,$str,$ra_line,$f_res,$rh_hits,$keep) = @_;

	if ( $keep eq 'all' )
	{
		printf $f_res "$str\n";
	}
	elsif ( $keep eq 'single' )
	{
		$$rh_hits{$$ra_line[0]} = ( !defined($$rh_hits{$$ra_line[0]}) ) ? $str : "MULTIPLEHITS";
	}
	elsif ( $keep eq 'best' )
	{
		if ( !defined($$rh_hits{$$ra_line[0]}) )
		{
			$$rh_hits{$$ra_line[0]} = $str;
			return;
		}
		my @a_curr = split(/\t/,$str);
		my @a_prev = split(/\t/,$$rh_hits{$$ra_line[0]});
		if ( $a_prev[11] < $a_curr[11] )
		{
			$$rh_hits{$$ra_line[0]} = $str;
		}
		elsif ( $a_prev[11] == $a_curr[11] )
		{
			$$rh_hits{$$ra_line[0]} .= "\tMULTIPLEHITS";
		}
	}
	return;
}

sub Usage
{
print<<END
$0
    --help	this message
[Mandatory]
    --lib           fastafile sequences to be mapped
    --reference     fastafile reference sequences
    --outfile       filename  result file
    --program       string    program to perform the mapping [soap,blastn]
    --program_param string    parameters associated to the mapping software
	
[Optional]
    --keep  string            [best] (all|single|best) filter out mapping result ('all' keeps all hits, 'single' only hits of read mapped once and 'best' keeps reads when the best hit has a best score than other hits)
    --cfg	filename          configuration file
    --overwrite               flag to redo the computation even when the expected results are found
    --debug
    
END
}
