use strict;
my @A_BLACKLIST = ();
my $TIMEOUT = 5;

sub BlackList
{
    my $ra_services = shift;

    my @a_services_clean = ();
    foreach my $o_service (@{$ra_services})
    {
        my $uri = $o_service->authority;
        next if (grep (/^$uri$/, @A_BLACKLIST));
        push(@a_services_clean, $o_service);
    }
    return \@a_services_clean;
}

sub BIOSConnectRepository
{
    my $dbconfig = shift;
    my $dbd      = shift;
    my $old_nl   = $/;
    $/ = "\n";
    my $o_param_dbconfig = New ParamParser($dbconfig);
    $o_param_dbconfig->SelectNameSpace($dbd . '_');

    my $dbname = $o_param_dbconfig->Get('dbname');
    my $dbhost = $o_param_dbconfig->Get('dbhost');
    my $dbport = $o_param_dbconfig->Get('dbport');
    my $dbuser = $o_param_dbconfig->Get('dbuser');
    my $dbpass = $o_param_dbconfig->Get('dbpass');

    $o_param_dbconfig->SelectNameSpace();

    my $data_source;
    $data_source = "dbi:Pg:database=$dbname;host=$dbhost;port=$dbport"    if ($dbd eq 'pgsql');
    $data_source = "dbi:mysql:database=$dbname;host=$dbhost;port=$dbport" if ($dbd eq 'mysql');
    $data_source = "dbi:SQLite:dbname=$dbname"                            if ($dbd eq 'sqlite');

    my $o_dbh = DBI->connect($data_source, $dbuser, $dbpass) or &UnlockAndCroak($DBI::errstr);
    $/ = $old_nl;
    return ($o_dbh, $dbname);
}

sub BIOSDisconnectRepository
{
    my $o_dbh = shift;
    $o_dbh->disconnect;
    return;
}

sub BIOSLoadCfg
{
    my $o_param = shift;
    my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    $dirprg = "." if ($dirprg eq "");
    my $main_cfg = $ENV{BIOS} . "/site/cfg/bios.cfg";
    &UnlockAndCroak("Error reading main configuration file>$main_cfg<\n") if (!-e $main_cfg);
    $o_param->Update($main_cfg, 'O');
    my $install_dir = $o_param->Get('install_dir');
    my $http_root = $o_param->Get('http_root');
    $o_param->SetBehaviour('use_substitution_table');
    $o_param->SetSubstitution('%i', \$install_dir);
    $o_param->SetSubstitution('%h', \$http_root);
    $o_param->Update($main_cfg, 'O');    #la les substitutions seront effectives !

    return $o_param;
}

sub BiomobyGetResourcesByServiceName
{
    my $o_moby_instance = shift;
    my $service_name    = shift;
    my $auth_uri        = shift;

    my %h_request = (serviceName => "$service_name");
    $h_request{authURI} = $auth_uri if (defined $auth_uri);

    my ($ra_service_instance, $r) = $o_moby_instance->findService(%h_request);

    $ra_service_instance = &BlackList($ra_service_instance);
    
#    print '<pre>';print Dumper $ra_service_instance;print '</pre>';
    &Carp::croak("ERROR : no ressource found for $service_name in current registry\n")
      if ($#{$ra_service_instance} < 0);
    return $ra_service_instance;
}

sub BiomobyGetBIOSProviders
{
    my $o_moby_instance  = shift;
    my @a_auth_uris      = $o_moby_instance->retrieveServiceProviders();
    my @a_bios_providers = ();
    foreach my $auth_uri (@a_auth_uris)
    {
    	next if ($auth_uri !~ /^bios\./);
	push(@a_bios_providers, $auth_uri) if (($auth_uri =~ /^bios\./) && !(grep (/^$auth_uri$/, @A_BLACKLIST)) && (&__IsBIOSProviderAlive($auth_uri) == 1));
    }
    return \@a_bios_providers;
}

sub __IsBIOSProviderAlive
{
	my $auth_uri = shift;
	my $cgi = 'http://' . $auth_uri . '/playmoby/cgi/index.cgi';
	$LWP::Simple::ua->timeout($TIMEOUT);
	my $cgi_ok = LWP::Simple::get($cgi);
	my $alive = 0;
	$alive = 1 if (defined $cgi_ok);
	return $alive;
}

sub BiomobyExecuteResources
{
    my $o_moby_instance     = shift;
    my $ra_service_instance = shift;
    my $rh_data             = shift;

#	use Data::Dumper;
#	print STDERR "o_moby_instance\n";
#	print STDERR Dumper $o_moby_instance;
#	print STDERR "ra_service_instance\n";
#	print STDERR Dumper $ra_service_instance;
#	print STDERR "rh_data\n";
#	print STDERR Dumper $rh_data;
	
    my %h_erroneous_uri = ();
    my %h_results;
    my $ra_bios_providers = &BiomobyGetBIOSProviders($o_moby_instance);
    
    foreach my $o_service_instance (@{$ra_service_instance})
    {
	my $auth_uri = $o_service_instance->authority;
#	print STDERR "auth_uri=$auth_uri\n";
	if (&__IsBIOSProviderAlive($auth_uri) == 0)
	{
		$h_erroneous_uri{$auth_uri} = 'PROVIDER IS DOWN';
		next;
	}
	
	my $o_service_invocation;

        my $wsdl = $o_moby_instance->retrieveService($o_service_instance);


	my $o_service_invocation;
	
        if ($o_service_instance->category eq 'moby-async')
        {
            eval {$o_service_invocation = MOBY::Async::Service->new(service => $wsdl)};
        }
        else
        {
            eval {$o_service_invocation = MOBY::Client::Service->new(service => $wsdl)};
        }

        if ($@)
        {	
#	    print STDERR "error=$@\n";
	    $h_erroneous_uri{$auth_uri} = "SERVICE INVOCATION FAILED $@";
	    next;
        }
	else
	{
        	my $result;

        	my %h_input = (Input => {1 => $rh_data});

        	eval {$result = $o_service_invocation->enumerated_execute(%h_input)};

        	if ($@)
        	{
#			print STDERR "error2=$@";
        	    $h_erroneous_uri{$auth_uri} = "SERVICE EXECUTION FAILED $@";
		    next;
        	}
#		print STDERR "result=$result\n";
        	$h_results{$auth_uri} = $result;    
#		print STDERR "h_results1\n";
#		print STDERR Dumper \%h_results;
	}
    }

#    print STDERR "h_results2\n";
#    print STDERR Dumper \%h_results;
    return (\%h_results,\%h_erroneous_uri);
}

sub BiomobyConnectRegistry
{
    my $cfg = shift;

    &Carp::croak("ERROR : BioMOBY registry configuration file >$cfg< does not exist \n") if (!-e $cfg);

    my $o_pp = New ParamParser("$cfg");
    $ENV{'MOBY_URI'}    = $o_pp->Get("mobycentral_uri");
    $ENV{'MOBY_SERVER'} = $o_pp->Get("mobycentral_url");

    my $o_moby_instance = MOBY::Client::Central->new();

    return ($o_pp, $o_moby_instance);

}

sub BIOSAuthentication
{
    my $o_param = shift;

    my $authentified_user = 0;

    my $login = $o_param->Get('login');
    my $password = $o_param->Get('password');
    
    if ($login =~ /^enc:(\S+)/)
    {
    	$login = &DecodeHomeMade($1);
    }
    if ($password =~ /^enc:(\S+)/)
    {
    	$password = &DecodeHomeMade($1);
    }

    my $o_auth;
    if (-e $o_param->Get('htpassword_file'))
    {
        $o_auth = New Auth(
                           {
                            login    => $login,
                            password => $password,
                            file     => $o_param->Get('htpassword_file'),
                           }
                           );
        $authentified_user = $o_auth->CheckAuth();
    }
    
    if ($authentified_user)
    {
    	$o_param->Set('login',$login);
    }
    return ($authentified_user, $o_auth, $o_param);
}

sub EncodeHomeMade
{
    my $mdp = shift;
    my $str = Convert::UU::uuencode($mdp);
    $str =~ s/^begin 644 uuencode.uu\s//o;
    $str =~ s/`\send\s$//o;
    $str =~ s/([^A-Za-z0-9])/sprintf("%%%02X", ord($1))/seg;
    $str = reverse($str);
    $str =~ s/%/:/g;
    return $str;
}

sub DecodeHomeMade
{
    my $encoded_substr = shift;
    $encoded_substr =~ s/:/%/g;
    $encoded_substr = reverse($encoded_substr);
    $encoded_substr =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
    $encoded_substr = "begin 644 uuencode.uu\n$encoded_substr\`\nend\n";

    my $str = Convert::UU::uudecode($encoded_substr);
    return $str;
}




sub BIOSCheckLibraryAccess
{
    my $o_param           = shift;
    my $ra_library_ids    = shift;
    my $authentified_user = shift;

    $o_param->AssertDefined('dbd');
    my $dbd = $o_param->Get('dbd');

    my $main_cfg = $ENV{BIOS} . "/site/cfg/bios.cfg";
    my ($o_dbh, $dbname) = &BIOSConnectRepository($main_cfg, $dbd);

    my $bios_user_id;
    $bios_user_id = $o_param->Get('login') if ($authentified_user);

    my $library_table = 'library';
    $library_table = $dbname . '.' . $library_table if ($dbd =~ /mysql/i);
    my $in_lib = join("','", @{$ra_library_ids});
    my $library_id_field = "$library_table.library_id";

    #my $sql_query ="SELECT $library_id_field FROM $library_table WHERE $library_id_field IN ('$in_lib') AND $library_id_field IN (SELECT DISTINCT access_control.library_id FROM access_control WHERE bios_user_id != '$bios_user_id')";
    my $sql_query = "SELECT $library_id_field FROM $library_table WHERE $library_id_field IN ('$in_lib') AND $library_id_field  IN (SELECT DISTINCT access_control.library_id FROM access_control "
    			. "WHERE bios_user_id != '$bios_user_id' "
			. "AND access_control.library_id NOT IN (SELECT DISTINCT access_control.library_id FROM access_control WHERE ( bios_user_id = '$bios_user_id')))";
			
    print STDERR "CheckLibraryAccess: $sql_query\n" if ($o_param->IsDefined('verbose'));
    my $sth = $o_dbh->prepare($sql_query);
    $sth->execute();
    my $count = $sth->rows;
    if ($count > 0)
    {
        my $message = "ERROR : You don\'t have access privilege to the $count following libraries:\n\n";
        while (my $rh_libraries = $sth->fetchrow_hashref)
        {
            $message .= "\t" . $rh_libraries->{library_id} . "\n";
        }
        &Carp::croak("\n\n$message\n\n");
    }
    return;
}

sub Cat
{
    my $file       = shift;
    my $exclude    = shift;
    my $content    = '';
    my $o_input_fh = new IO::File($file) || Carp::croak("ERROR: $file not found");
    if (defined($o_input_fh))
    {
        while (my $line = <$o_input_fh>)
        {
            if (defined $exclude)
            {
                $content .= $line if ($line !~ /$exclude/);
            }
            else
            {
                $content .= $line;
            }
        }
        $o_input_fh->close;
    }

    return $content;
}

sub Save
{
    my $data = shift;
    my $file = shift;

    my $o_output_fh = new IO::File(">$file") || Carp::croak("ERROR: cannot write in $file");
    if (defined($o_output_fh))
    {
        print $o_output_fh $data;
        $o_output_fh->close;
    }

}

sub Lock
{
    my $o_param   = shift;
    my $lock_file = &GetLockFileName($o_param);
    if (-e $lock_file)
    {
        Carp::croak("ERROR: LOCK FILE FOUND\n\n $0 is already running. Please wait until it finish first\n\n");
    }
    else
    {
        my $fh_lock_file = new IO::File(">$lock_file") || Carp::croak("ERROR: cannot open >$lock_file<\n\n");
        print $fh_lock_file $$;
        $fh_lock_file->close;
    }
}

sub Unlock
{
    my $o_param   = shift;
    my $lock_file = &GetLockFileName($o_param);
    unlink($lock_file) if (-e $lock_file);
}

sub UnlockAndCroak
{
    my $o_param = shift;
    my $message = shift();
    &Unlock($o_param);
    Carp::croak($message);
}

sub GetLockFileName
{
    my $o_param = shift;
    my $dbd     = $o_param->Get('dbd');
    $o_param->SelectNameSpace($dbd . '_');
    my $dbname = $o_param->Get('dbname');
    my $dbhost = $o_param->Get('dbhost');
    $o_param->SelectNameSpace();
    my $lock_file = '/tmp/' . basename($0) . '.' . $dbname . '.' . $dbhost . '.lock';
    return $lock_file;
}

###########################################################
#   GET EXPRESSION
###########################################################

sub PrintExpressionFile
{
    my $rh_libraries = shift;
    my $o_param      = shift;
    if ($o_param->Get('outformat') eq 'iantxml')
    {
        &PrintXMLExpressionFile($rh_libraries, $o_param);
    }
    elsif ($o_param->Get('outformat') eq 'table')
    {
        &PrintTableExpressionFile($rh_libraries, $o_param);
    }
}

sub PrintXMLExpressionFile
{
    my $rh_libraries = shift;
    my $o_param      = shift;

    my $db_entry_str = "<?xml version=\"1.0\"?>\n<entry_db accession=\"\">";
    if (defined $rh_libraries->{pool1} && defined $rh_libraries->{pool1})
    {
	    foreach my $id (keys %{$rh_libraries->{pool1}->{reads}})
	    {
        	my $o_entry = New iANT::Entry();
        	$o_entry->SetId($id);

        	my $accession1 = $rh_libraries->{pool1}->{reads}->{$id}->{accession};
        	my $accession2 = $rh_libraries->{pool2}->{reads}->{$id}->{accession};
        	$accession1 =~ s/^#//;
        	$accession1 =~ s/#$//;
        	$accession2 =~ s/^#//;
        	$accession2 =~ s/#$//;

        	my $accession = $accession1;
        	if ($accession eq '')
        	{
        	    $accession = $accession2;
        	}
        	elsif (($accession2 ne '') && ($accession2 ne $accession1))
        	{
        	    $accession .= '#' . $accession2;
        	}

        	$o_entry->SetAccession($accession);

        	my $product1 = $rh_libraries->{pool1}->{reads}->{$id}->{definition};
        	my $product2 = $rh_libraries->{pool2}->{reads}->{$id}->{definition};


        	my $o_entry_product1 = New iANT::Entry::Definition({id => 'product', content => $product1}) if ($product1 ne '' );
        	my $o_entry_product2 = New iANT::Entry::Definition({id => 'product', content => $product2}) if (($product2 ne '' ) && ($product2 ne $product1));
        	$o_entry->SetDefinition($o_entry_product1) if (defined $o_entry_product1);
        	$o_entry->SetDefinition($o_entry_product2) if (defined $o_entry_product2);
        	my $o_db_feature = New iANT::DB::Feature();

        	# New library count
        	my $pattern_pool1 = New iANT::Pattern();
        	$pattern_pool1->SetDefinition($rh_libraries->{pool1}->{id});
        	$pattern_pool1->SetType("pool");
        	my $property_count_pool1 = New iANT::Pattern::Property(
                                	     {"id" => "count", "value" => $rh_libraries->{pool1}->{reads}->{$id}->{count_raw}});
        	$pattern_pool1->SetProperty($property_count_pool1);
        	$o_db_feature->SetPattern($pattern_pool1);

        	my $pattern_pool2 = New iANT::Pattern();
        	$pattern_pool2->SetDefinition($rh_libraries->{pool2}->{id});
        	$pattern_pool2->SetType("pool");
        	my $property_count_pool2 = New iANT::Pattern::Property(
                                	     {"id" => "count", "value" => $rh_libraries->{pool2}->{reads}->{$id}->{count_raw}});
        	$pattern_pool2->SetProperty($property_count_pool2);
        	$o_db_feature->SetPattern($pattern_pool2);

        	$o_entry->SetEntryFeatures($o_db_feature);

        	my $o_twig = XML::Twig->new();
        	$o_twig->set_pretty_print('indented');
        	$o_twig->parse($o_entry->GetXml) if (defined $o_entry);
        	$db_entry_str .= $o_twig->sprint() if (defined $o_entry);

	    }
    }
    else
    {
    	my $o_entry = New iANT::Entry();
	my $o_db_feature = New iANT::DB::Feature();

	my $generic_done = 0;
    	foreach my $pool_id (keys %{$rh_libraries})
	{
		foreach my $id (keys %{$rh_libraries->{$pool_id}->{reads}})
		{
			if (!  $generic_done)
			{
				$o_entry->SetId($id);
	        		my $accession = $rh_libraries->{$pool_id}->{reads}->{$id}->{accession};
        			$o_entry->SetAccession($accession);
				my $url = $rh_libraries->{$pool_id}->{reads}->{$id}->{url};
				$o_entry->SetXmlFile($url);
        			my $product = $rh_libraries->{$pool_id}->{reads}->{$id}->{definition};
        			my $o_entry_product = New iANT::Entry::Definition({id => 'product', content => $product}) if ($product ne '' );
	        		$o_entry->SetDefinition($o_entry_product) if (defined $o_entry_product);

				$generic_done = 1;
			}
        		
        		# New library count
			next if (!defined $rh_libraries->{$pool_id}->{id});
#			print STDERR "ID $id\t$rh_libraries->{$pool_id}->{id}\t$rh_libraries->{$pool_id}->{reads}->{$id}->{count_raw}\n";
        		my $pattern_pool = New iANT::Pattern();
        		$pattern_pool->SetDefinition($rh_libraries->{$pool_id}->{id});
        		$pattern_pool->SetType("library");
        		my $property_count_pool = New iANT::Pattern::Property({"id" => "count", "value" => $rh_libraries->{$pool_id}->{reads}->{$id}->{count_raw}});
        		$pattern_pool->SetProperty($property_count_pool);
			#print $pattern_pool->GetXml . "\n";
        		$o_db_feature->SetPattern($pattern_pool);
		}
	}
    
			$o_entry->SetEntryFeatures($o_db_feature);
    
    my $o_twig = XML::Twig->new();
        		$o_twig->set_pretty_print('indented');
        		$o_twig->parse($o_entry->GetXml) if (defined $o_entry);
        		$db_entry_str .= $o_twig->sprint() if (defined $o_entry); 
    }
   
    $db_entry_str .= "</entry_db>";

    my $output = $o_param->Get('expression_outfile');
    my $f_xml = new IO::File(">$output") or Carp::croak "Error: fail to create $output.\n";
    print $f_xml $db_entry_str;
    $f_xml->close();
}

sub PrintTableExpressionFile
{
    my $rh_libraries = shift;
    my $o_param      = shift;

    my $table = "gene\tpool1\tpool2";

    #$table .= "\tlength\tdefinition";
    $table .= "\n";

    my %h_total = (pool1 => 0, pool2 => 0);

    foreach my $id (keys %{$rh_libraries->{pool1}->{reads}})
    {
        my $accession1 = $rh_libraries->{pool1}->{reads}->{$id}->{accession};
        my $accession2 = $rh_libraries->{pool2}->{reads}->{$id}->{accession};

        $accession1 =~ s/^#//;
        $accession1 =~ s/#$//;
        $accession2 =~ s/^#//;
        $accession2 =~ s/#$//;

        my $accession = $accession1;

        if ($accession eq '')
        {
            $accession = $accession2;
        }
        elsif (($accession2 ne '') && ($accession2 ne $accession1))
        {
            $accession .= '#' . $accession2;
        }

        $table .= $id . '#' . $accession;

        $table .= "\t" . $rh_libraries->{pool1}->{reads}->{$id}->{count_raw};
        $h_total{pool1} += $rh_libraries->{pool1}->{reads}->{$id}->{count_raw};
        $table .= "\t" . $rh_libraries->{pool2}->{reads}->{$id}->{count_raw};
        $h_total{pool2} += $rh_libraries->{pool2}->{reads}->{$id}->{count_raw};

#$table .= "\t" . $rh_libraries->{pool1}->{reads}->{$id}->{length};
#$table .= "\t" . $rh_libraries->{pool1}->{reads}->{$id}->{description} . '#' . $rh_libraries->{pool2}->{reads}->{$id}->{description};

        $table .= "\n";
    }
    $table .= "TOTAL\t$h_total{pool1}\t$h_total{pool2}\n";

    my $output = $o_param->Get('expression_outfile');
    my $f_table = new IO::File(">$output") or Carp::croak "Error: fail to create $output.\n";
    print $f_table $table;
    $f_table->close();
}

sub PrintCollectionFile
{
    my $rh_libraries = shift;
    my $o_param      = shift;


    my $o_updated_collection = New BIOS::Collection();
	my $not_empty = 0;
    foreach my $pool_id (keys %{$rh_libraries})
    {
    	my $o_class = $rh_libraries->{$pool_id}->{object};
	    #&Carp::croak("ERROR: Class element not found for $pool_id") 
	    next if (!defined $o_class);
	    $o_class->SetCount_raw($rh_libraries->{$pool_id}->{count_raw});
	    $o_updated_collection->SetClass($o_class);
	    $not_empty = 1;
   }

   if ($not_empty)
   {
        $o_updated_collection->Dump('>' . $o_param->Get('collection_outfile'));
   }
}

sub GetLibraries
{
    my $o_param           = shift;
    my $authentified_user = shift;
    my %h_libraries;

    my $o_collection = New BIOS::Collection($o_param->Get('pools_collection_file'));
#    &Carp::croak("ERROR: Your pools_collection_file have to contain 2 classes\n")
 
   if ($#{$o_collection->GetClasses} < 1)
   {
   	$h_libraries{whole} = {};
	$h_libraries{whole}->{libraries} = [];
	
   	my $ra_libraries = $o_collection->GetLibraries;
	my $count = 0;
	foreach my $o_library (@{$ra_libraries})
	{
		$count ++;
		$h_libraries{"library$count"} = {
                           object    => undef,
                           id        => $o_library->GetId,
                           libraries => [$o_library->GetId],
                           count_raw => 0,
                           reads     => {}
                           };
		push (@{$h_libraries{whole}->{libraries}},$o_library->GetId);
	}
   }
   else
   {
    my $pool1_id = 'pool1';

    $pool1_id = ${$o_collection->GetClasses}[0]->GetId() if (defined $o_collection->GetClasses);

    my $pool2_id = 'pool2';

    $pool2_id = ${$o_collection->GetClasses}[1]->GetId() if (defined $o_collection->GetClasses);

    my $ra_libraries_1 = &GetLibraryIdsFromClass(${$o_collection->GetClasses}[0]);
    my $ra_libraries_2 = &GetLibraryIdsFromClass(${$o_collection->GetClasses}[1]);

    $h_libraries{pool1} = {
                           object    => ${$o_collection->GetClasses}[0],
                           id        => $pool1_id,
                           libraries => $ra_libraries_1,
                           count_raw => 0,
                           reads     => {}
                           };
    $h_libraries{pool2} = {
                           object    => ${$o_collection->GetClasses}[1],
                           id        => $pool2_id,
                           libraries => $ra_libraries_2,
                           count_raw => 0,
                           reads     => {}
                           };
    $h_libraries{whole} = {libraries => [@{$ra_libraries_1}, @{$ra_libraries_2}]};
    }

    &BIOSCheckLibraryAccess($o_param, $h_libraries{whole}->{libraries}, $authentified_user);
    return \%h_libraries;
}

sub GetLibraryIdsFromClass
{

    #my $o_collection = shift;
    my $o_class = shift;
    my @a_ids;

    my $ra_libraries;

    #    if (defined $o_collection->GetClasses)
    if (defined $o_class)
    {

        #$ra_libraries = ${$o_collection->GetClasses}[0]->GetLibraries;
        $ra_libraries = $o_class->GetLibraries;
    }
    else
    {

        #        $ra_libraries = $o_collection->GetLibraries;
        &Carp::croak("ERROR: No class defined\n");
    }

    foreach my $o_library (@{$ra_libraries})
    {
        push(@a_ids, $o_library->GetId);
    }
    return (\@a_ids);
}

sub GetSpeciesCode
{
    my $organism_xml_file = shift;
    my $o_organism        = New iANT::Organism($organism_xml_file);

    my $o_species = $o_organism->GetSpecies();
    &Carp::croak("ERROR: No species defined in your xml file\n") if (!defined $o_species);

    my $species_code = $o_species->GetCode();
    &Carp::croak("ERROR: No species code defined in your xml file\n")
      if (!defined $species_code || ($species_code eq ''));

    return $species_code;

}

sub GetValuesFromParameter
{
    my $parameter_value = shift();
    my @a_values = ();
    
    my $list_separators = '[,\s\|;]+';
    
    if (-e $parameter_value)
    {
        my $fh_values = new IO::File($parameter_value) || Carp::croak("ERROR: $! >$parameter_value<");
        if (defined $fh_values)
        {
            while (my $value = <$fh_values>)
            {
                chomp $value;
                push (@a_values,$value) if ($value ne '');
            }
            $fh_values->close;
        }
    }
    elsif ($parameter_value =~ /$list_separators/ )
    {
        @a_values = split (/$list_separators/,$parameter_value);
    }
    else
    {
       @a_values = ($parameter_value);
    }
    
    return \@a_values;
}

1;
