#!/usr/bin/perl


# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

configure.pl - Compendium Install/Configuration script

=head1 SYNOPSIS

configure.pl [--pasa_path <dir>] [--tgicl_path <dir>] [--framedp_path <dir>] [--verbose]

=head1 DESCRIPTION

When you install compendium environnement for the first time you have to configure it.
The script configure.pl will modify config files to tune the pipeline compendium according to your system.


% tcsh
% ./configure.pl  [--pasa_path <dir>] [--tgicl_path <dir>] [--framedp_path <dir>] [--verbose]

    Optional
        --pasa_path       : path to the repository where PASA is installed, if it's not in the repository compendium
        --tgicl_path      : path to the repository where tgicl++ is installed, if it's not in the repository compendium
		--framedp_path    : path to the repository where FrameDP is installed, if it's not in the repository compendium
		--verbose
=head1 AUTHORS

 Marion.Verdenaud@toulouse.inra.fr

=head1 SUBROUTINES

=cut

use strict;
my ($dirprg, $nameprg);
use Term::ANSIColor;

BEGIN
{

    ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    
    $dirprg = '.' if (! (defined $dirprg) || ($dirprg eq ''));
    
    if (-d "$dirprg/lib")
    {
        unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../lib")
    {
        unshift(@INC, "$dirprg/../lib");
    }

}

use ParamParser;
use File::Which qw(which);
use Cwd 'abs_path';

my $O_PARAM;
my @A_MANDATORY_BINARIES = ('ln','sed') ;
my %H_SCRIPTS = ("pasa_script"=>"Launch_PASA_pipeline.pl","pasa_conf"=>"pasa.alignAssembly.Template.txt","pasa_annotation_cleaner"=>"pasa_annotation_cleaner.pl","seqclean"=>"seqclean","tgicl"=>"tgicl++","framedp_bin"=>"FrameDP.pl","framedp_conf"=>"FrameDP.cfg",
"velveth"=>"velveth","velvetg"=>"velvetg","oases"=>"oases","bios_mapreads"=>"bios_mapreads.pl","bios_m8like2compendiumreport"=>"bios_m8like2compendiumreport.pl");
$dirprg = abs_path($dirprg);

MAIN:
{

    $O_PARAM = New ParamParser('GETOPTLONG', ("pasa_script_path=s","pasa_conf_path=s","seqclean_path=s","tgicl_path=s","framedp_bin_path=s","framedp_conf_path=s",
	"velveth_path=s","velvetg_path=s","oases_path=s","bios_mapreads_path=s","bios_m8like2compendiumreport_path=s","paraloop_path=s","pasa_annotation_cleaner=s","verbose","h"));
	$O_PARAM->SetUsage(my $usage = sub {&Usage();});



	if ($O_PARAM->IsDefined('h'))
	{
		$O_PARAM->Usage();
	}


	my %h_mask_prg;
	#cas particulier pour paraloop:
	if ($O_PARAM->IsDefined('paraloop_path'))
	{
		$O_PARAM->AssertDirExists('paraloop_path');
		$h_mask_prg{"#paraloop_mask#"}=$O_PARAM->Get('paraloop_path');
		print STDERR color 'bold green';
		print STDERR "[OK] ".uc("paraloop")." repository = ".$O_PARAM->Get('paraloop_path')."\n\n" if ($O_PARAM->IsDefined('verbose'));
		print STDERR color 'reset';
	}
	else
	{
		my $paraloop_path=`printenv PARALOOP`;
		chomp $paraloop_path;
		if (! defined $paraloop_path || $paraloop_path eq "")
		{
			my $cmd = "find -L $dirprg -type f -name \\*paraloop.pl 2> /dev/null";
			print $cmd."\n" if ($O_PARAM->IsDefined('verbose'));
			my $script_path=`$cmd`;
			($paraloop_path) = $script_path =~ /(.+)\/bin\/paraloop.pl/;
		}

		if (defined $paraloop_path && $paraloop_path ne "")
		{
			$h_mask_prg{"#paraloop_mask#"}=$paraloop_path;
			print STDERR color 'bold green';
			print STDERR "[OK] ".uc("paraloop")." repository = $paraloop_path\n\n" if ($O_PARAM->IsDefined('verbose'));
			print STDERR color 'reset';
		}
		else
		{
			print STDERR color 'red';
			print STDERR "The parameter paraloop_path  or the environment variable PARALOOP have to be defined\n";
			print STDERR color 'reset';
			exit;
		}
	}


	foreach my $script (keys %H_SCRIPTS)
	{
		my $find_dir=$dirprg;
		if ($O_PARAM->IsDefined("$script"."_path"))
		{
			$O_PARAM->AssertDirExists("$script"."_path");
			$find_dir = abs_path($O_PARAM->Get("$script"."_path"));
		}

		my $cmd = "find -L $find_dir -type f -name \\*".$H_SCRIPTS{$script} ." 2> /dev/null";
		print $cmd."\n" if ($O_PARAM->IsDefined('verbose'));
		my $script_path=`$cmd`;
		
		$script_path = which($H_SCRIPTS{$script}) if  ($script_path eq "");
		chomp $script_path;
		if ($script_path eq "")
		{
			print STDERR color 'yellow';
			print STDERR uc($script) ." : ".$H_SCRIPTS{$script}."  is not well installed, you will not be able to use it via the compendium pipeline.\n If you want to, please install correctly ".uc($script)." in the repository compendium or give the path where ".uc($script)." is installed.\n\n";
			print STDERR color 'reset';
		}
		else
		{
			my $val =$H_SCRIPTS{$script};
			$val =~ s/\//\\\//g;
			$val =~ s/\+/\\\+/g;
			my ($script_mask) = $script_path =~ /(.*)$val/;
			print STDERR color 'bold green';
			print STDERR "[OK] ".uc($script)." repository = $script_mask\n\n" if ($O_PARAM->IsDefined('verbose'));
			print STDERR color 'reset';
			$h_mask_prg{"#$script"."_mask#"}=$script_mask;
		}
	}


	my $rh_mask_binaries = &CheckConfig();
	
	foreach my $k (keys (%$rh_mask_binaries))
	{
		$h_mask_prg{$k}=$rh_mask_binaries->{$k};
	}

	my $prg_file_mask = $dirprg . '/cfg/compendium_programs.cfg.mask';
    die("ERROR: $prg_file_mask not found") if (!-e $prg_file_mask);
    my $prg_file = $dirprg . '/cfg/compendium_programs.cfg';
    print STDERR "Substitute $prg_file_mask\n";
    &Substitute($prg_file_mask, $prg_file,\%h_mask_prg);

	$prg_file_mask = $dirprg . '/cfg/compendium_pipeline.cfg.mask';
    die("ERROR: $prg_file_mask not found") if (!-e $prg_file_mask);
    $prg_file = $dirprg . '/cfg/compendium_pipeline.cfg';
    print STDERR "Substitute $prg_file_mask\n";
    &Substitute($prg_file_mask, $prg_file,\%h_mask_prg);
}


sub CheckConfig
{
	my $config_ok = 1;
    my %h_mask_binaries = ();

	foreach my $mandatory_binary (@A_MANDATORY_BINARIES)
	{
		my $exe_path = which($mandatory_binary);

		$h_mask_binaries{ '#'.$mandatory_binary . '_mask#'} = $exe_path;
	
		if (!defined($exe_path) || ($exe_path eq ''))
		{
			print STDERR color 'red';
			print STDERR "\nERROR CONFIGURATION: $mandatory_binary binary can not be found on your system\n\n";
			print STDERR color 'reset';
			$config_ok = 0;
		}
		else
		{
			print STDERR color 'bold green';
			print STDERR "[OK] $mandatory_binary = $exe_path\n" if ($O_PARAM->IsDefined('verbose'));
			print STDERR color 'reset';

		}
	}
	
	if ($config_ok == 0)
    {
		print STDERR color 'bold red';
    	print STDERR ("\n\nPRE-REQUISITE MODULES/BINARIES ARE MISSING. CONFIGURATION FAILED\n\n\n");
		print STDERR color 'reset';
		exit;
    }
    else
    {
		print STDERR color 'bold green';
    	print STDERR "\n\nPRE-REQUISITE MODULES/BINARIES CONFIGURATION IS OK\n\n\n";
		print STDERR color 'reset';
    }

    return \%h_mask_binaries;

}
=head2 procedure Substitute

 Title        : Substitute

=cut

sub Substitute
{
    my ($in, $out, $rh_mask_binaries) = (@_);

    my $in_fh  = new IO::File($in);
    my $out_fh = new IO::File(">$out");
    die("$!: $in")  if (!defined $in_fh);
    die("$!: $out") if (!defined $out_fh);

    while (my $line = <$in_fh>)
    {
        chomp $line;

        if (defined $rh_mask_binaries)
		{
			foreach my $mask_binary (keys %{$rh_mask_binaries})
    		{
				$line =~ s/$mask_binary/$rh_mask_binaries->{$mask_binary}/g;
			}
		}

        print $out_fh $line . "\n";
    }

    $in_fh->close;
    $out_fh->close;

}

sub Usage
{
	 print STDOUT "

Usage:


     $0 

    Optional :\n";

	foreach my $script (keys %H_SCRIPTS)
	{
		print STDOUT "\t--$script"."_path : path to the repository $script where ".$H_SCRIPTS{$script}." is , if it's not compendium's repository \n";
	}
	print STDOUT "\t--h : display usage\n\t--verbose\n";

}
