#!/usr/bin/perl


BEGIN
{
    my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    unshift(@INC, "$dirprg/../../../lib/");
}

our $VERSION = do {my @r = (q$Rev: 3005 $ =~ /\d+/g); $r[0]};

use warnings;
use strict;
use IO::File;
use ParamParser;

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',("prg=s","in=s","param=s","stdout=s","stderr=s","parse=s"));
    $o_param->SetUsage(my $usage = sub {&Usage();});
	$o_param->AssertDefined("in");
	$o_param->AssertDefined("stdout");
	$o_param->AssertFileExists("prg");
	$o_param->AssertFullPath("prg");

	my $prg   = $o_param->Get('prg');
	my $in    = $o_param->Get('in');
	my $out   = $o_param->Get('stdout');

	my $cde = " $prg $in ";
	if ( $o_param->IsDefined('param') )
	{
		my $param = $o_param->Get('param');
		$cde .= " $cde $param ";
	}
	$cde .= " > $out ";
	my $stde = undef;
	if ( $o_param->IsDefined('stderr') )
	{
		$stde = $o_param->Get('stderr');
		$cde = "( $cde ) 2> $stde";
	}
	system($cde);
	
	if ( $o_param->IsDefined('parse') )	# TGICL++ Simplification step parsing
	{
		my $prefix = $o_param->Get('parse');
		&TGICLParsing($in,"$out.parsed",$prefix);
		$cde = "rm $in $in.cap.*; gzip $out $out.parsed";
		system($cde);
		unlink $stde if ( defined($stde) && -z $stde );
	}
}



sub TGICLParsing
{
my($in,$out,$head_id)=@_;

	my $f_out = new IO::File($out,"w") or die "Can't create >$out<\n"; 
	my $str_file_contig        = '';
	my $str_file_contig_qual   = '';
	my $str_file_contig_report = '';
	my ($id) = $in =~  /.+\/\w*CL(.+)/;
	my $f_ace = undef;
	if ( defined($f_ace=new IO::File("$in.cap.ace")) )
	{
		my @a_fa  = <$f_ace>;
		my @a_pls = grep(/^CO/,@a_fa);
		my $pls   = $#a_pls+1;
		my $co=0;
		my $bq=0;
		my $new=0;
		my $seq="";
		my $af=0;
		foreach my $line (@a_fa)
		{
			if ($line =~/^CO\s(Contig\d*)\s/)
			{
				my $contig=$1;
				$co=1;
				$seq="";
				$str_file_contig	    .= $head_id."_$id";
				$str_file_contig_qual	.= $head_id."_$id";
				$str_file_contig_report	.= $head_id."_$id";
				if ($pls > 1)
				{
					$str_file_contig 	    .= "_$contig";
					$str_file_contig_qual	.= "_$contig";
					$str_file_contig_report	.= "_$contig";
				}
				$str_file_contig        .= "\t";
				$str_file_contig_qual   .= "\tQUAL\t";
				$str_file_contig_report .= "\tREPORT\t";
			}
			elsif (($line =~ /^[ACGTN*]+$/) && ($co == 1))
			{
				chomp $line;
				$line =~ s/\*//g;
				$seq .= $line
			}
			elsif ($co == 1)
			{
				$str_file_contig .= "CONSENSUS\t$seq\n";
				$co=0;
			}
			elsif($line =~ /^BQ/)
			{
				$bq=1;
			}
			elsif (($line =~ /^(\d+\s*)+/) && ($bq == 1))
			{
				chomp $line;
				$str_file_contig_qual .= "$line#";
			}
			elsif ($bq == 1)
			{
				$str_file_contig_qual .= "\n";
				$bq=0;
			}
			elsif ($line =~/^AF\s(\S+)\s/)
			{
				my $id=$1;
				$str_file_contig_report .= "$id ";
				$af=1;
			}
			elsif ( $af )
			{
				$str_file_contig_report =~ s/\s$//;
				$str_file_contig_report .= "\n";
				$af=0;
			}
		}
		$f_ace->close;
	}

	my $f_singlets = undef;
	if ( defined($f_singlets = new IO::File("$in.cap.singlets")) )
	{
		my $first = 1;
		while (my $line = <$f_singlets>)
		{
			chomp($line);
			if ($line =~ />(\S*)/)
			{
				my $id=$1;
				$str_file_contig .= "\n" if ( ! $first );	
				$str_file_contig .= "$id\tSINGLETS\t";
				$str_file_contig_report .= "$id\tREPORT\t$id\n";
				$first=0;
			}
			else
			{
				$str_file_contig .= $line;
			}
		}
		$f_singlets->close;
	}
	print $f_out "$id\n";
	print $f_out "$str_file_contig\n";
	print $f_out "$str_file_contig_qual\n";
	print $f_out "$str_file_contig_report\n";
	$f_out->close;
}



sub Usage
{
   print STDOUT <<END;

	Usage:

    $0 --in <fasta file> --stdout <filename> --prg filename [--stderr <filename>] [--param <quoted string>]
	
	Mandatory :
		in     : the input fasta file
		stdout : a filename for stdout
		prg    : complete path of cap3

	Optional :
		param  : cap3 parameters
		stderr : a filename for stderr
END
}

